
#include <stdio.h>
#include "sp.h"


/* **********************************************************************

status[v]  indica se o vertice v est na rvore, na orla ou fora

parent[v] contm o vertice que  pai de v na rvore (arco candidato de v)

fringeLink[v] contm a ligao (o nmero do) prximo vrtice da orla

distance[v] contm a distncia da origem a v

fringeWgt[v] contm o peso do arco candidato de v 

********************************************************************** */


// -------------------------------------------------------------------
// Shortest Path (algoritmo de Dijkstra)
void sp(Graph g, int n, int s, int d)       // g com vertices de 1 a n
{
  int status[MAXVERTICES];
  int parent[MAXVERTICES];
  int fringeLink[MAXVERTICES];
  int distance[MAXVERTICES];
  Node *p;         // para atravessar as listas de adjacncia
  int x,y;
  int fringeList;  // o primeiro vrtice da orla
  int stuck;
  int prev;        // auxilar para a remoo dum vrtice da orla


  /* inicializao */
  x = s; 
  fringeList = 0;
  for (y = 1 ; y <= n ; y++) status[y] = UNSEEN;      
  status[s] = INTREE;
  distance[s] = 0;
  stuck = 0;

  while (x!=d && !stuck) {

    /* atravessar lista de adjacncias de x */
    for (p=g[x]; p; p=p->next) {
      y = p->vertex;
      if (status[y]==FRINGE && distance[x]+p->weight < distance[y]) {
	/* substituir arco candidato de y por xy */
	parent[y] = x;
	distance[y] = distance[x]+p->weight;
      }
      else if (status[y]==UNSEEN) {
	/* y est na orla; xy  arco candidato */
	status[y] = FRINGE;
	fringeLink[y] = fringeList;
	fringeList = y;
	parent[y] = x;
	distance[y] = distance[x]+p->weight;
      }
    } /* for */

    /* escolher prximo vrtice e arco para a rvore */
    if (fringeList==0) 
      stuck = 1;
    else {
      prev = 0;
      x = fringeList;
      for (y=fringeList; fringeLink[y]; y=fringeLink[y]) 
      /* escolher o arco que coloca o n da orla a menor distncia da origem */
	if (distance[fringeLink[y]] < distance[x]) {
	  x = fringeLink[y];
	  prev = y;
	}
      /* remover o arco candidato (x) da orla */
      if (x==fringeList) 
	fringeList = fringeLink[x];
      else fringeLink[prev] = fringeLink[x];
      fringeLink[x] = 0;
      status[x] = INTREE;
    }
  } /* while */


  /* mostra caminho e distncia */
  if (!stuck) {
    for(x=d;x!=s;x=parent[x])
      printf("\n\t%d--%d", x, parent[x]);
    
    printf("\n\n\tDistancia = %d\n", distance[d]);
  } 
  else printf("\n Nao existe caminho entre %d e %d ! \n",s,d);


}


// --------------------------------------------- ----------------------
// Minimum Spanning Tree (algoritmo de Prim)
void mst(Graph g, int n)                     // g com vertices de 1 a n
{
  int status[MAXVERTICES];
  int parent[MAXVERTICES];
  int fringeLink[MAXVERTICES];
  int fringeWgt[MAXVERTICES];
  Node *p;         // para atravessar as listas de adjacncia
  int x,y;
  int fringeList;  // o primeiro vrtice da orla
  int edgeCount;   // nmero de arcos na rvore
  int stuck;
  int prev;        // auxilar para a remoo dum vrtice da orla
  int sum;

  /* inicializao */

  x = 1; 
  status[1] = INTREE;
  for (y = 2 ; y <= n ; y++) status[y] = UNSEEN;
  edgeCount = 0;
  fringeList = 0;
  stuck = 0;
  
  while (edgeCount<n-1 && !stuck) {

    /* atravessar lista de adjacncias de x */
    for (p=g[x]; p; p=p->next) {
      y = p->vertex;
      if (status[y]==FRINGE && p->weight<fringeWgt[y]) {
	/* substituir arco candidato de y por xy */
	parent[y] = x;
	fringeWgt[y] = p->weight;
      }
      else if (status[y]==UNSEEN) {
	/* y est na orla; xy  arco candidato */
	status[y] = FRINGE;
	fringeLink[y] = fringeList;
	fringeList = y;
	parent[y] = x;
	fringeWgt[y] = p->weight;
      }
    } /* for */

    /* escolher prximo vrtice e arco para a rvore */
    if (fringeList==0) 
      stuck = 1;
    else {
      prev = 0;
      x = fringeList;
      for (y=fringeList; fringeLink[y]; y=fringeLink[y]) 
	/* escolher o arco candidato de menor peso */
	if (fringeWgt[fringeLink[y]]<fringeWgt[x]) {
	  x = fringeLink[y];
	  prev = y;
	}
      /* remover o arco candidato (x) da orla */
      if (x==fringeList) 
	fringeList = fringeLink[x];
      else fringeLink[prev] = fringeLink[x];
      fringeLink[x] = 0;
      status[x] = INTREE;
      edgeCount++;
    }
  } /* while */

  /* mostrar rvore e peso total */
  if (!stuck) {
    for (x=2, sum=0 ; x<=n ; x++) {
      printf("\n\t%d--%d, peso %d", x, parent[x], fringeWgt[x]);
      sum+=fringeWgt[x];  
    }
    printf("\n\n\tPeso total = %d\n", sum);
  }
  else printf("\n Grafo nao ligado ! \n");
}


